#! /usr/bin/env perl

# Find specified LaTeX-style \use*{*} in standard input,
# adding options as necessary.  (First occurrence only.)

use 5.006_001;
use strict;
use warnings;

our $prefix = '\use';
our $prefix_qr = quotemeta($prefix);
our @use_directives = ();

# Scan @ARGV for directives.  For each, recognize "type[options]{name}",
# store [ string, regexp, [ options ] ] into @use_directives.
foreach my $use (@ARGV) {
  if ($use =~ m{^([^\[\{]*)(\[([^\]]*)\])?((\{([^\}]*)\})?)}) {
    my ($type, $options, $namepart) = ($1, $3, quotemeta($4));
    my $re =
      qr{([^%]*$prefix_qr$type(\s*%.*$)*)(\[([^\]]*)\])?((\s*%.*$)*$namepart)};
    $options = "" unless defined($options);
    push(@use_directives, [ $use, $re, [ split(",", $options) ] ]);
  }
}

# Read standard input.
my $text = "";
while (<STDIN>) {
  $text .= $_;
}

# Find the document class, split into before and after.
my ($pre_doc, $doc_class, $post_doc);
if ($text =~
    m{\\documentclass(\s*(%.*$)?)*
      (\[[^\]]*\])?(\s*(%.*$)?)*
      \{[^\}]*\}
      \s*(\s*%.*$ \s*^)*}xm) {
  ($pre_doc, $doc_class, $post_doc) = ($`, $&, $');
} else {
  # No document class found.
  print $text;
  exit(0);
}

# Replace directives, add if necessary.
foreach my $d (@use_directives) {
  my ($string, $re, $options_ref) = @$d;
  my @options = ();
  @options = @$options_ref if ($options_ref);

  # Match against regexp.
  if ($post_doc =~ m{^$re}m) {
    # Match, merge options.
    my ($pre, $use, $current_options, $namepart, $post) = ($`, $1, $4, $5, $');
    my $new_options;
    $current_options = "" unless defined($current_options);
    if (($current_options eq "") && (! @options)) {
      $new_options = "";
    } elsif (! @options) {
      $new_options = $current_options;
    } elsif ($current_options eq "") {
      $new_options = join(",", @options);
    } else {
      $new_options = $current_options;
      foreach my $opt (@options) {
	$new_options .= ",$opt" unless ($current_options =~ m{(^|,)$opt(,|$)});
      }
    }

    # New text with merged options.
    $new_options = "[$new_options]" unless ($new_options eq "");
    $post_doc = "$pre$use$new_options$namepart$post";
  } else {
    # No match of our regexp, add the directive after document class.
    $doc_class .= "$prefix$string\n";
  }
}

# Print result.
print "$pre_doc$doc_class$post_doc";
